@@ -629,6 +629,13 @@ void ModelData<RealType>::convertCovariateToDense(const IdType covariate) {
     X.getColumn(index).convertColumnToDense(getNumberOfRows());
 }
 
+template <typename RealType>
+void ModelData<RealType>::convertAllCovariatesToDense(int length) {
+    for (int index = 0; index < getNumberOfColumns(); ++index) {
+        X.getColumn(index).convertColumnToDense(length);
+    }
+}
+
 template <typename RealType>
 void ModelData<RealType>::setOffsetCovariate(const IdType covariate) {
     int index;
