@@ -23,25 +23,27 @@ public:
 			long inSeed,
     	    loggers::ProgressLoggerPtr _logger,
 	    	loggers::ErrorHandlerPtr _error,
-			std::vector<real>* wtsExclude = NULL);
+			std::vector<double>* wtsExclude,
+			std::vector<double>* wtsOriginal);
 
 	virtual ~CrossValidationSelector();
 
 	void permute();
-	
+
  	void reseed();
 
-	void getWeights(int batch, std::vector<real>& weights);
+	void getWeights(int batch, std::vector<double>& weights);
+
+	void getComplement(std::vector<double>& weights);
 
-	void getComplement(std::vector<real>& weights);
-	
 	AbstractSelector* clone() const;
 
 private:
 	int fold;
 	std::vector<int> permutation;
 	std::vector<int> intervalStart;
-	std::vector<real>* weightsExclude;
+	std::vector<double>* weightsExclude;
+	std::vector<double>* weightsOriginal;
 };
 
 } // namespace
