@@ -61,7 +61,8 @@ public:
 			double* iBeta,
 			const double* iY) = 0; // pure virtual
 
-	virtual void setWeights(double* inWeights, bool useCrossValidation) = 0; // pure virtual
+	//virtual void setWeights(double* inWeights, bool useCrossValidation) = 0; // pure virtual
+	virtual void setWeights(double* inWeights, double* cenWeights, bool useCrossValidation) = 0; // pure virtual
 
 	virtual void computeGradientAndHessian(int index, double *ogradient,
 			double *ohessian, bool useWeights) = 0; // pure virtual
